
#use IO::Handle;

$help_message = "
HTMLSECTION_VERSION

\thtmlsection [-options] [files ...]

htmlsection is a perl script to insert section number in HTML. When you use
htmlsection, you don't have to manage section, table, figure, program-list,
and reference numbers and you don't have to make the table of contents,
table index, figure index, and program-list index.

* Insert section number in HTML and make links to the section.
* Make the table of contents.
* Insert table, figure, and program-list number and make links.
* Make the index of tables, figures, and program-lists.
* Insert reference number and make links.

If you want to see more detail information, See htmlsection(1),
documents in HTMLSECTION_PREFIX/share/doc/htmlsection, and comments
at the head of htmlsection script. And see,

\thttp://web.ffn.ne.jp/~hsakai/myfreesoft/htmlsection.html

htmlsection Copyright (C) 2001-2003 SAKAI Hiroaki.
All Rights Reserved.

";

$max_depth = 6;
$spacer = 1;
@prev_space = (70, 50, 30);
@next_space = (30, 25, 20);

for ($i = 0; $i < $max_depth; $i++) {
	$section[$i] = 0;
}

# This is for temporary files. But, temporary files are not used.
# 
# -tmpdir [directry]
#     Specify the directry to make temporary file. If you don't specify
#     -tmpdir option, environment variable TMPDIR will be used.
# 
#     TMPDIR
#     Directry to make the temporary file. If you don't specify environment
#     variable TMPDIR and -tmpdir option, /tmp will be used.
# 
#$tmpdir = $ENV{"TMPDIR"};
#if ($tmpdir eq "") { $tmpdir = "/tmp"; }

$lang = $ENV{"LANG"};
if (($lang eq "ja_JP.EUC") || ($lang eq "japanese")) {
	$lang = "japanese";
} else {
	$lang = "english";
}

$table_section_num  = 1;
$figure_section_num = 1;
$list_section_num   = 1;
$tablenum  = 0;
$figurenum = 0;
$listnum   = 0;

$refnum = 0;
$ref_section_num = 1;
$ref_number_flag = 1;
$termnum = 0;
$term_section_num = 1;
$term_number_flag = 1;
$notenum = 0;
$note_section_num = 1;
$note_number_flag = 1;

$toc_filename    = "";
$table_filename  = "";
$figure_filename = "";
$list_filename   = "";
$ref_filename    = "";
$term_filename   = "";
$note_filename   = "";
$output_filename = "";

$toc_section = 4;

while ($_ = shift(@ARGV)) {

	if (($_ eq "-h") || ($_ eq "-help")) {
		print "$help_message";
		exit (0);
	}

	elsif ($_ eq "-max-depth") { $max_depth = shift(@ARGV); }
	elsif ($_ eq "-spacer") { $spacer = 1; }
	elsif ($_ eq "-no-spacer") { $spacer = 0; }
	elsif ($_ eq "-english" ) { $lang = "english"; }
	elsif ($_ eq "-japanese") { $lang = "japanese"; }
	elsif ($_ eq "-start-section") {
		$sec = shift(@ARGV);
		for ($i = 0; $i < $max_depth; $i++) {
			($n) = $sec =~ /^(\d*)/;
			$sec =~ s/^(\d*)(\D*)//;
			if ($n ne "") { $section[$i] = $n; }
		}
	}
	elsif ($_ eq "-toc-section"   ) { $toc_section        = shift(@ARGV); }
	elsif ($_ eq "-table-section" ) { $table_section_num  = shift(@ARGV); }
	elsif ($_ eq "-figure-section") { $figure_section_num = shift(@ARGV); }
	elsif ($_ eq "-list-section"  ) { $list_section_num   = shift(@ARGV); }
	elsif ($_ eq "-start-table" ) { $tablenum  = shift(@ARGV); }
	elsif ($_ eq "-start-figure") { $figurenum = shift(@ARGV); }
	elsif ($_ eq "-start-list"  ) { $listnum   = shift(@ARGV); }
	elsif ($_ eq "-start-ref"   ) { $refnum    = shift(@ARGV); }
	elsif ($_ eq "-start-term"  ) { $termnum   = shift(@ARGV); }
	elsif ($_ eq "-start-note"  ) { $notenum   = shift(@ARGV); }
	elsif ($_ eq "-ref-number"    ) { $ref_number_flag  = 1; }
	elsif ($_ eq "-term-number"   ) { $term_number_flag = 1; }
	elsif ($_ eq "-note-number"   ) { $note_number_flag = 1; }
	elsif ($_ eq "-no-ref-number" ) { $ref_number_flag  = 0; }
	elsif ($_ eq "-no-term-number") { $term_number_flag = 0; }
	elsif ($_ eq "-no-note-number") { $note_number_flag = 0; }
	elsif ($_ eq "-toc-file"    ) { $toc_filename    = shift(@ARGV); }
	elsif ($_ eq "-table-file"  ) { $table_filename  = shift(@ARGV); }
	elsif ($_ eq "-figure-file" ) { $figure_filename = shift(@ARGV); }
	elsif ($_ eq "-list-file"   ) { $list_filename   = shift(@ARGV); }
	elsif ($_ eq "-ref-file"    ) { $ref_filename    = shift(@ARGV); }
	elsif ($_ eq "-term-file"   ) { $term_filename   = shift(@ARGV); }
	elsif ($_ eq "-note-file"   ) { $note_filename   = shift(@ARGV); }
	elsif ($_ eq "-o"           ) { $output_filename = shift(@ARGV); }
# This is for temporary files. But, temporary files are not used.
#	elsif ($_ eq "-tmpdir") { $tmpdir = shift(@ARGV); }
}

# This is for temporary files. But, temporary files are not used.
##system ("rm -f $tmpdir/htmlsection.tmp.*");
#$r = rand();
#($filename) = $r =~ /(........)$/;
#$filename = "$tmpdir/htmlsection.tmp.$filename";

if ($lang eq "japanese") {
	$table_string = "ɽ";
	$figure_string = "";
	$list_string = "ꥹ";
	$reference_string = "ʸ";
	$term_string = "Ѹ";
	$note_string = "";
	$toc_string = "ܼ";
	$table_index_string = "ɽܼ";
	$figure_index_string = "ܼ";
	$list_index_string = "ꥹܼ";
	$reference_index_string = "ʸ";
	$term_index_string = "Ѹ";
	$note_index_string = "";
} else {
	$table_string = "table";
	$figure_string = "figure";
	$list_string = "list";
	$reference_string = "Reference";
	$term_string = "Term";
	$note_string = "Note";
	$toc_string = "Table of Contents";
	$table_index_string = "Table Index";
	$figure_index_string = "Figure Index";
	$list_index_string = "List Index";
	$reference_index_string = "References";
	$term_index_string = "Terms";
	$note_index_string = "Notes";
}

@table_toc  = ();
@figure_toc = ();
@list_toc   = ();
@ref_toc    = ();
@term_toc   = ();
@note_toc   = ();

# This is for temporary files. But, temporary files are not used.
#$filename_num = 0;
#$filename_num++;
#open (WFILE, "> $filename.$filename_num");

# This is for debug
#WFILE->autoflush(1);

$no_space = 0;
$toc_num = 0;
$table_num = 0;
$figure_num = 0;
$list_num = 0;
$ref_num = 0;
$toc = "";
$depth = 0;
$line = 0;
$section_number_flag = 1;
@lines = ();

while(<>) {
	$line++;
	$prev = "";
	$next = "";

	$sec_flag = 0;

	if (/section/i) {
		for ($i = 0; $i < $max_depth; $i++) {
			$sub = "sub" x $i;
			$sec = $sub . "section";
			while (($tag) = /(<$sec[^>]*?>.*?<\/$sec[^>]*?>)/i) {
				$sec_flag = 1;
#				if (($i > $depth) && ($i - $depth > 1)) {
#					die "line$line: invalid section.\n";
#				}

				($title) = $tag =~ /<$sec[^>]*?>(.*?)<\/$sec[^>]*?>/i;
				($n) = $tag =~ /<$sec[^>]*?number\s*=\s*([^\s>]*)[\s>]/i;
				if ($n =~ /\".*\"/) { ($n) = $n =~ /\"(.*)\"/; }
				$number_flag = 1;
				if    ($n =~ /^yes$/i) { $number_flag = 1; }
				elsif ($n =~ /^on$/i ) { $number_flag = 1; }
				elsif ($n =~ /^no$/i ) { $number_flag = 0; }
				elsif ($n =~ /^off$/i) { $number_flag = 0; }
				$section_number_flag = $number_flag;

				if ($number_flag) {
					$section[$i]++;
					for ($j = $i+1; $j < $max_depth; $j++) {
						$section[$j] = 0;
					}

					$t = "";
					for ($j = 0; $j <= $i; $j++) {
						$t = "$t$section[$j]";
						if ($j < $i) {
							$t = "$t.";
						}
					}
					$title = "$t $title";
				} else {
					$toc_num++;
					$t = "toc$toc_num";
				}

				($name) = $tag =~ /<$sec[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
				if ($name =~ /\".*\"/) { ($name) = $name =~ /\"(.*)\"/; }
				$name =~ s/\//\_/g;
				if ($name eq "") { $name = "$t"; }
				$name = "$sec\_$name";
				if ($sectionref[$i]{$name} ne "") {
					die "line$line: duplicate section name: $name\n";
				}
				$sectionref[$i]{$name} = $t;
				$sectionref_title[$i]{$name} = $title;
				$t = "<a name=\"$name\">$title</a>";
				$n = $i+1;
				unless (s/\Q$tag/<h$n>$t<\/h$n>/i) {
					die "line$line: fail to parse: $_\n";
				}

				if (($toc_section == 0) || ($n <= $toc_section)) {
					if ($n > $depth) {
						$toc = $toc . "<ul compact>\n" x ($n - $depth);
					} elsif ($n < $depth) {
						$toc = $toc . "</ul>\n" x ($depth - $n);
					}
					$toc = $toc . "<li><a href=\"#$name\">$title</a></li>\n";
					$depth = $n;

				}

				if ($n <= $table_section_num ) { $tablenum  = 0; }
				if ($n <= $figure_section_num) { $figurenum = 0; }
				if ($n <= $list_section_num  ) { $listnum   = 0; }

				if (($prev_space[$i] != 0) && ($no_space == 0)) {
					$prev = "<spacer type=vertical size=$prev_space[$i]>\n";
				}
				if ($next_space[$i] != 0) {
					$next = "<spacer type=vertical size=$next_space[$i]>\n";
					$no_space = 1;
				}
			}
		}
	}

	if ($sec_flag == 0) {
		unless (/^\s*$/) {
			$no_space = 0;
		}
	}

	while (($tag) = /(<tablereference[^>]*?>.*?<\/tablereference[^>]*?>)/i) {
		($title) = $tag =~ /<tablereference[^>]*?>(.*?)<\/tablereference[^>]*?>/i;
		($num) = $tag =~ /<tablereference[^>]*?number\s*=\s*([^\s>]*)[\s>]/i;
		if ($num =~ /\".*\"/) { ($num) = $num =~ /\"(.*)\"/; }
		$number_flag = 1;
		if    ($num =~ /^yes$/i) { $number_flag = 1; }
		elsif ($num =~ /^on$/i ) { $number_flag = 1; }
		elsif ($num =~ /^no$/i ) { $number_flag = 0; }
		elsif ($num =~ /^off$/i) { $number_flag = 0; }
		($name) = $tag =~ /<tablereference[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
		if ($name =~ /\".*\"/) { ($name) = $name =~ /\"(.*)\"/; }
		$name =~ s/\//\_/g;
		if ($number_flag) {
			$tablenum++;
			$r = "";
			for ($i = 0; $i < $table_section_num; $i++) {
				if ($section[$i] < 1) { last; }
				$r = "$r$section[$i].";
			}
			$r = "$r$tablenum";
			if ($name eq "") { $name = "$r"; }
			if ($section_number_flag == 0) { $r = "$tablenum"; }
		} else {
			$table_num++;
			$r = "";
			if ($name eq "") { $name = "num$table_num"; }
		}
		$name = "tbl_$name";
		$r = "$table_string$r";
		$ref = "$r: $title";
		if ($tableref{$name} ne "") {
			die "line$line: duplicate table name: $name\n";
		}
		$tableref{$name} = $r;
		$tableref_title{$name} = $ref;
		$t = "<li><a href=\"#$name\">$ref</a>\n";
		push(@table_toc, $t);
		unless (s/\Q$tag/<a name=\"$name\">$ref<\/a>/i) {
			die "line$line: fail to parse: $_\n";
		}
	}

	while (($tag) = /(<figurereference[^>]*?>.*?<\/figurereference[^>]*?>)/i) {
		($title) = $tag =~ /<figurereference[^>]*?>(.*?)<\/figurereference[^>]*?>/i;
		($num) = $tag =~ /<figurereference[^>]*?number\s*=\s*([^\s>]*)[\s>]/i;
		if ($num =~ /\".*\"/) { ($num) = $num =~ /\"(.*)\"/; }
		$number_flag = 1;
		if    ($num =~ /^yes$/i) { $number_flag = 1; }
		elsif ($num =~ /^on$/i ) { $number_flag = 1; }
		elsif ($num =~ /^no$/i ) { $number_flag = 0; }
		elsif ($num =~ /^off$/i) { $number_flag = 0; }
		($name) = $tag =~ /<figurereference[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
		if ($name =~ /\".*\"/) { ($name) = $name =~ /\"(.*)\"/; }
		$name =~ s/\//\_/g;
		if ($number_flag) {
			$figurenum++;
			$r = "";
			for ($i = 0; $i < $figure_section_num; $i++) {
				if ($section[$i] < 1) { last; }
				$r = "$r$section[$i].";
			}
			$r = "$r$figurenum";
			if ($name eq "") { $name = "$r"; }
			if ($section_number_flag == 0) { $r = "$figurenum"; }
		} else {
			$figure_num++;
			$r = "";
			if ($name eq "") { $name = "num$figure_num"; }
		}
		$name = "fig_$name";
		$r = "$figure_string$r";
		$ref = "$r: $title";
		if ($figureref{$name} ne "") {
			die "line$line: duplicate figure name: $name\n";
		}
		$figureref{$name} = $r;
		$figureref_title{$name} = $ref;
		$t = "<li><a href=\"#$name\">$ref</a>\n";
		push(@figure_toc, $t);
		unless (s/\Q$tag/<a name=\"$name\">$ref<\/a>/i) {
			die "line$line: fail to parse: $_\n";
		}
	}

	while (($tag) = /(<listreference[^>]*?>.*?<\/listreference[^>]*?>)/i) {
		($title) = $tag =~ /<listreference[^>]*?>(.*?)<\/listreference[^>]*?>/i;
		($num) = $tag =~ /<listreference[^>]*?number\s*=\s*([^\s>]*)[\s>]/i;
		if ($num =~ /\".*\"/) { ($num) = $num =~ /\"(.*)\"/; }
		$number_flag = 1;
		if    ($num =~ /^yes$/i) { $number_flag = 1; }
		elsif ($num =~ /^on$/i ) { $number_flag = 1; }
		elsif ($num =~ /^no$/i ) { $number_flag = 0; }
		elsif ($num =~ /^off$/i) { $number_flag = 0; }
		($name) = $tag =~ /<listreference[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
		if ($name =~ /\".*\"/) { ($name) = $name =~ /\"(.*)\"/; }
		$name =~ s/\//\_/g;
		if ($number_flag) {
			$listnum++;
			$r = "";
			for ($i = 0; $i < $list_section_num; $i++) {
				if ($section[$i] < 1) { last; }
				$r = "$r$section[$i].";
			}
			$r = "$r$listnum";
			if ($name eq "") { $name = "$r"; }
			if ($section_number_flag == 0) { $r = "$listnum"; }
		} else {
			$list_num++;
			$r = "";
			if ($name eq "") { $name = "num$list_num"; }
		}
		$name = "lst_$name";
		$r = "$list_string$r";
		$ref = "$r: $title";
		if ($listref{$name} ne "") {
			die "line$line: duplicate list name: $name\n";
		}
		$listref{$name} = $r;
		$listref_title{$name} = $ref;
		$t = "<li><a href=\"#$name\">$ref</a>\n";
		push(@list_toc, $t);
		unless (s/\Q$tag/<a name=\"$name\">$ref<\/a>/i) {
			die "line$line: fail to parse: $_\n";
		}
	}

	if (($spacer) && ($prev ne "")) {
# This is for temporary files. But, temporary files are not used.
#		print WFILE $prev;
		push(@lines, $prev);
	}
# This is for temporary files. But, temporary files are not used.
#	print WFILE $_;
	push(@lines, $_);
	if (($spacer) && ($next ne "")) {
# This is for temporary files. But, temporary files are not used.
#		print WFILE $next;
		push(@lines, $next);
	}
}

$toc = $toc . "</ul>\n" x ($depth);

# This is for temporary files. But, temporary files are not used.
#close (WFILE);
#open (RFILE, "< $filename.$filename_num");
#$filename_num++;
#open (WFILE, "> $filename.$filename_num");

$line = 0;
@lines2 = ();

# This is for temporary files. But, temporary files are not used.
#while (<RFILE>)
while ($_ = shift(@lines))
{
	$line++;

	if (/sectionref/i) {
		for ($i = 0; $i < $max_depth; $i++) {
			$sub = "sub" x $i;
			$sec = $sub . "sectionref";

			while (($tag) = /(<$sec[^>]*?>)/i) {
				($t) = $tag =~ /<$sec[^>]*?title\s*=\s*([^\s>]*)[\s>]/i;
				if ($t =~ /\".*\"/) { ($t) = $t =~ /\"(.*)\"/; }
				($n) = $tag =~ /<$sec[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
				if ($n =~ /\".*\"/) { ($n) = $n =~ /\"(.*)\"/; }
				$title_flag = 1;
				if    ($t =~ /^yes$/i) { $title_flag = 1; }
				elsif ($t =~ /^on$/i ) { $title_flag = 1; }
				elsif ($t =~ /^no$/i ) { $title_flag = 0; }
				elsif ($t =~ /^off$/i) { $title_flag = 0; }
				$n =~ s/\//\_/g;
				if ($n eq "") { die "line$line: $_:no name.\n"; }
				$name = "$sub" . "section\_$n";
				if ($title_flag) { $ref = $sectionref_title[$i]{$name}; }
				else { $ref = $sectionref[$i]{$name}; }
				if ($ref eq "") { die "line$line: $_$n: not found.\n"; }
				unless (s/\Q$tag/<a href=\"\#$name\">$ref<\/a>/i) {
					die "line$line: fail to parse: $_\n";
				}
			}
		}
	}

	while (($tag) = /(<tableref[^>]*?>)/i) {
		($t) = $tag =~ /<tableref[^>]*?title\s*=\s*([^\s>]*)[\s>]/i;
		if ($t =~ /\".*\"/) { ($t) = $t =~ /\"(.*)\"/; }
		($n) = $tag =~ /<tableref[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
		if ($n =~ /\".*\"/) { ($n) = $n =~ /\"(.*)\"/; }
		$title_flag = 0;
		if    ($t =~ /^yes$/i) { $title_flag = 1; }
		elsif ($t =~ /^on$/i ) { $title_flag = 1; }
		elsif ($t =~ /^no$/i ) { $title_flag = 0; }
		elsif ($t =~ /^off$/i) { $title_flag = 0; }
		$n =~ s/\//\_/g;
		if ($n eq "") { die "line$line: $_:no name.\n"; }
		$name = "tbl_$n";
		if ($title_flag) { $ref = $tableref_title{$name}; }
		else { $ref = $tableref{$name}; }
		if ($ref eq "") { die "line$line: $_$n: not found.\n"; }
		unless (s/\Q$tag/<a href=\"\#$name\">$ref<\/a>/i) {
			die "line$line: fail to parse: $_\n";
		}
	}
	while (($tag) = /(<figureref[^>]*?>)/i) {
		($t) = $tag =~ /<figureref[^>]*?title\s*=\s*([^\s>]*)[\s>]/i;
		if ($t =~ /\".*\"/) { ($t) = $t =~ /\"(.*)\"/; }
		($n) = $tag =~ /<figureref[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
		if ($n =~ /\".*\"/) { ($n) = $n =~ /\"(.*)\"/; }
		$title_flag = 0;
		if    ($t =~ /^yes$/i) { $title_flag = 1; }
		elsif ($t =~ /^on$/i ) { $title_flag = 1; }
		elsif ($t =~ /^no$/i ) { $title_flag = 0; }
		elsif ($t =~ /^off$/i) { $title_flag = 0; }
		$n =~ s/\//\_/g;
		if ($n eq "") { die "line$line: $_:no name.\n"; }
		$name = "fig_$n";
		if ($title_flag) { $ref = $figureref_title{$name}; }
		else { $ref = $figureref{$name}; }
		if ($ref eq "") { die "line$line: $_$n: not found.\n"; }
		unless (s/\Q$tag/<a href=\"\#$name\">$ref<\/a>/i) {
			die "line$line: fail to parse: $_\n";
		}
	}
	while (($tag) = /(<listref[^>]*?>)/i) {
		($t) = $tag =~ /<listref[^>]*?title\s*=\s*([^\s>]*)[\s>]/i;
		if ($t =~ /\".*\"/) { ($t) = $t =~ /\"(.*)\"/; }
		($n) = $tag =~ /<listref[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
		if ($n =~ /\".*\"/) { ($n) = $n =~ /\"(.*)\"/; }
		$title_flag = 0;
		if    ($t =~ /^yes$/i) { $title_flag = 1; }
		elsif ($t =~ /^on$/i ) { $title_flag = 1; }
		elsif ($t =~ /^no$/i ) { $title_flag = 0; }
		elsif ($t =~ /^off$/i) { $title_flag = 0; }
		$n =~ s/\//\_/g;
		if ($n eq "") { die "line$line: $_:no name.\n"; }
		$name = "lst_$n";
		if ($title_flag) { $ref = $listref_title{$name}; }
		else { $ref = $listref{$name}; }
		if ($ref eq "") { die "line$line: $_$n: not found.\n"; }
		unless (s/\Q$tag/<a href=\"\#$name\">$ref<\/a>/i) {
			die "line$line: fail to parse: $_\n";
		}
	}
# This is for temporary files. But, temporary files are not used.
#	print WFILE $_;
	push(@lines2, $_);
}

# This is for temporary files. But, temporary files are not used.
#close (RFILE);
#close (WFILE);
#open (RFILE, "< $filename.$filename_num");
#$filename_num++;
#open (WFILE, "> $filename.$filename_num");

$line = 0;
@lines = @lines2;
@lines2 = ();

# This is for temporary files. But, temporary files are not used.
#while (<RFILE>)
while ($_ = shift(@lines))
{
	$line++;
	if (/<references>/i) {
		$refnum = 0;
		$noname_num = 0;
		s/<references>/<h2>$reference_index_string<\/h2>\n\n<ul>/i;
		while (!(/<\/references>/i)) {
			if (($tag) = /(<reference[^>]*?>)/i) {
				($t) = $tag =~ /<reference[^>]*?title\s*=\s*([^\s>]*)[\s>]/i;
				if ($t =~ /\".*\"/) { ($t) = $t =~ /\"(.*)\"/; }
				($n) = $tag =~ /<reference[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
				if ($n =~ /\".*\"/) { ($n) = $n =~ /\"(.*)\"/; }
				($m) = $tag =~ /<reference[^>]*?number\s*=\s*([^\s>]*)[\s>]/i;
				if ($m =~ /\".*\"/) { ($m) = $m =~ /\"(.*)\"/; }
				$title_flag = 0;
				if    ($t =~ /^yes$/i) { $title_flag = 1; }
				elsif ($t =~ /^on$/i ) { $title_flag = 1; }
				elsif ($t =~ /^no$/i ) { $title_flag = 0; }
				elsif ($t =~ /^off$/i) { $title_flag = 0; }
				$number_flag = $ref_number_flag;
				if    ($m =~ /^yes$/i) { $number_flag = 1; }
				elsif ($m =~ /^on$/i ) { $number_flag = 1; }
				elsif ($m =~ /^no$/i ) { $number_flag = 0; }
				elsif ($m =~ /^off$/i) { $number_flag = 0; }
				$n =~ s/\//\_/g;
				if ($n eq "") {
					$noname_num++;
					$n = "noname_${noname_num}";
				}
				$name = "ref_${ref_section_num}_$n";
				if (exists($ref{$name})) {
					$refnum = $ref{$name};
				} else {
					$refnum++;
					$ref{$name} = $refnum;
				}
				if ($number_flag) {
					if ($ref{$name} ne "") {
						$num = "[$ref{$name}]";
					} else {
						$num = "[*]";
					}
				} else {
					$num = "";
				}
				if ($title_flag) { $num = "$ref_string$num"; }
				unless (s/\Q$tag/<li><a name=\"$name\">$num<\/a>\&nbsp\;/i) {
					die "line$line: fail to parse: $_\n";
				}
			}
# This is for temporary files. But, temporary files are not used.
#			print WFILE $_;
#			unless ($_ = <RFILE>) { last; }
			push(@lines2, $_);
			push(@ref_toc, $_);
			unless ($_ = shift(@lines)) { last; }
		}
		s/<\/references>/<\/ul>/i;
		push(@ref_toc, $_);
		$refnum = 0;
		$ref_section_num++;
		%ref = ();
	} else {
		while (($tag) = /(<ref[^>]*?>)/i) {
			($t) = $tag =~ /<ref[^>]*?title\s*=\s*([^\s>]*)[\s>]/i;
			if ($t =~ /\".*\"/) { ($t) = $t =~ /\"(.*)\"/; }
			($n) = $tag =~ /<ref[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
			if ($n =~ /\".*\"/) { ($n) = $n =~ /\"(.*)\"/; }
			($m) = $tag =~ /<ref[^>]*?number\s*=\s*([^\s>]*)[\s>]/i;
			if ($m =~ /\".*\"/) { ($m) = $m =~ /\"(.*)\"/; }
			$title_flag = 0;
			if    ($t =~ /^yes$/i) { $title_flag = 1; }
			elsif ($t =~ /^on$/i ) { $title_flag = 1; }
			elsif ($t =~ /^no$/i ) { $title_flag = 0; }
			elsif ($t =~ /^off$/i) { $title_flag = 0; }
			$number_flag = $ref_number_flag;
			if    ($m =~ /^yes$/i) { $number_flag = 1; }
			elsif ($m =~ /^on$/i ) { $number_flag = 1; }
			elsif ($m =~ /^no$/i ) { $number_flag = 0; }
			elsif ($m =~ /^off$/i) { $number_flag = 0; }
			$n =~ s/\//\_/g;
			if ($n eq "") { die "line$line: $_:no name.\n"; }
			$name = "ref_${ref_section_num}_$n";
			if ($number_flag) {
				if (!exists($ref{$name})) {
					$refnum++;
					$ref{$name} = $refnum;
				}
				$num = "[$ref{$name}]";
			} else {
				$ref{$name} = "";
				$num = "";
			}
			if ($title_flag) { $num = "$ref_string$num"; }
			unless (s/\Q$tag/<a href=\"\#$name\">$num<\/a>/i) {
				die "line$line: fail to parse: $_\n";
			}
		}
	}
# This is for temporary files. But, temporary files are not used.
#	print WFILE $_;
	push(@lines2, $_);
}

# This is for temporary files. But, temporary files are not used.
#close (RFILE);
#close (WFILE);
#open (RFILE, "< $filename.$filename_num");
#$filename_num++;
#open (WFILE, "> $filename.$filename_num");

$line = 0;
@lines = @lines2;
@lines2 = ();

# This is for temporary files. But, temporary files are not used.
#while (<RFILE>)
while ($_ = shift(@lines))
{
	$line++;
	if (/<terms>/i) {
		$termnum = 0;
		$noname_num = 0;
		s/<terms>/<h2>$term_index_string<\/h2>\n\n<ul>/i;
		while (!(/<\/terms>/i)) {
			if (($tag) = /(<term[^>]*?>)/i) {
				($t) = $tag =~ /<term[^>]*?title\s*=\s*([^\s>]*)[\s>]/i;
				if ($t =~ /\".*\"/) { ($t) = $t =~ /\"(.*)\"/; }
				($n) = $tag =~ /<term[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
				if ($n =~ /\".*\"/) { ($n) = $n =~ /\"(.*)\"/; }
				($m) = $tag =~ /<term[^>]*?number\s*=\s*([^\s>]*)[\s>]/i;
				if ($m =~ /\".*\"/) { ($m) = $m =~ /\"(.*)\"/; }
				($w) = $tag =~ /<term[^>]*?word\s*=\s*([^\s>]*)[\s>]/i;
				if ($w =~ /\".*\"/) { ($w) = $w =~ /\"(.*)\"/; }
				$title_flag = 0;
				if    ($t =~ /^yes$/i) { $title_flag = 1; }
				elsif ($t =~ /^on$/i ) { $title_flag = 1; }
				elsif ($t =~ /^no$/i ) { $title_flag = 0; }
				elsif ($t =~ /^off$/i) { $title_flag = 0; }
				$number_flag = $term_number_flag;
				if    ($m =~ /^yes$/i) { $number_flag = 1; }
				elsif ($m =~ /^on$/i ) { $number_flag = 1; }
				elsif ($m =~ /^no$/i ) { $number_flag = 0; }
				elsif ($m =~ /^off$/i) { $number_flag = 0; }
				$n =~ s/\//\_/g;
				if ($n eq "") {
					$noname_num++;
					$n = "noname_${noname_num}";
				}
				$name = "term_${term_section_num}_$n";
				if (exists($term{$name})) {
					$termnum = $term{$name};
				} else {
					$termnum++;
					$term{$name} = $termnum;
				}
				if ($number_flag) {
					if ($term{$name} ne "") {
						$num = "($term{$name})";
					} else {
						$num = "(*)";
					}
				} else {
					$num = "";
				}
				if ($title_flag) { $num = "$term_string$num"; }
				if ($w eq "") { $w = $termword{$name}; }
				unless (s/\Q$tag/<li><a name=\"$name\">$num$w<\/a>\&nbsp\;/i) {
					die "line$line: fail to parse: $_\n";
				}
			}
# This is for temporary files. But, temporary files are not used.
#			print WFILE $_;
#			unless ($_ = <RFILE>) { last; }
			push(@lines2, $_);
			push(@term_toc, $_);
			unless ($_ = shift(@lines)) { last; }
		}
		s/<\/terms>/<\/ul>/i;
		push(@term_toc, $_);
		$termnum = 0;
		$term_section_num++;
		%term = ();
		%termword = ();
	} else {
			while (($tag) = /(<termref[^>]*?>.*?<\/termref[^>]*?>)/i) {
			($w) = $tag =~ /<termref[^>]*?>(.*?)<\/termref[^>]*?>/i;
			($t) = $tag =~ /<termref[^>]*?title\s*=\s*([^\s>]*)[\s>]/i;
			if ($t =~ /\".*\"/) { ($t) = $t =~ /\"(.*)\"/; }
			($n) = $tag =~ /<termref[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
			if ($n =~ /\".*\"/) { ($n) = $n =~ /\"(.*)\"/; }
			($m) = $tag =~ /<termref[^>]*?number\s*=\s*([^\s>]*)[\s>]/i;
			if ($m =~ /\".*\"/) { ($m) = $m =~ /\"(.*)\"/; }
			$title_flag = 0;
			if    ($t =~ /^yes$/i) { $title_flag = 1; }
			elsif ($t =~ /^on$/i ) { $title_flag = 1; }
			elsif ($t =~ /^no$/i ) { $title_flag = 0; }
			elsif ($t =~ /^off$/i) { $title_flag = 0; }
			$number_flag = $term_number_flag;
			if    ($m =~ /^yes$/i) { $number_flag = 1; }
			elsif ($m =~ /^on$/i ) { $number_flag = 1; }
			elsif ($m =~ /^no$/i ) { $number_flag = 0; }
			elsif ($m =~ /^off$/i) { $number_flag = 0; }
			$n =~ s/\//\_/g;
			if ($n eq "") { die "line$line: $_:no name.\n"; }
			$name = "term_${term_section_num}_$n";
			if ($number_flag) {
				if (!exists($term{$name})) {
					$termnum++;
					$term{$name} = $termnum;
					$termword{$name} = $w;
				}
				$num = "($term{$name})";
			} else {
				$term{$name} = "";
				$termword{$name} = "";
				$num = "";
			}
			if ($title_flag) { $num = "$term_string$num"; }
			unless (s/\Q$tag/<a href=\"\#$name\">$num$w<\/a>/i) {
				die "line$line: fail to parse: $_\n";
			}
		}
	}
# This is for temporary files. But, temporary files are not used.
#	print WFILE $_;
	push(@lines2, $_);
}

# This is for temporary files. But, temporary files are not used.
#close (RFILE);
#close (WFILE);
#open (RFILE, "< $filename.$filename_num");
#$filename_num++;
#open (WFILE, "> $filename.$filename_num");

$line = 0;
@lines = @lines2;
@lines2 = ();

# This is for temporary files. But, temporary files are not used.
#while (<RFILE>)
while ($_ = shift(@lines))
{
	$line++;
	if (/<notes>/i) {
		$notenum = 0;
		$noname_num = 0;
		s/<notes>/<h2>$note_index_string<\/h2>\n\n<ul>/i;
		while (!(/<\/notes>/i)) {
			if (($tag) = /(<note[^>]*?>)/i) {
				($t) = $tag =~ /<note[^>]*?title\s*=\s*([^\s>]*)[\s>]/i;
				if ($t =~ /\".*\"/) { ($t) = $t =~ /\"(.*)\"/; }
				($n) = $tag =~ /<note[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
				if ($n =~ /\".*\"/) { ($n) = $n =~ /\"(.*)\"/; }
				($m) = $tag =~ /<note[^>]*?number\s*=\s*([^\s>]*)[\s>]/i;
				if ($m =~ /\".*\"/) { ($m) = $m =~ /\"(.*)\"/; }
				$title_flag = 0;
				if    ($t =~ /^yes$/i) { $title_flag = 1; }
				elsif ($t =~ /^on$/i ) { $title_flag = 1; }
				elsif ($t =~ /^no$/i ) { $title_flag = 0; }
				elsif ($t =~ /^off$/i) { $title_flag = 0; }
				$number_flag = $note_number_flag;
				if    ($m =~ /^yes$/i) { $number_flag = 1; }
				elsif ($m =~ /^on$/i ) { $number_flag = 1; }
				elsif ($m =~ /^no$/i ) { $number_flag = 0; }
				elsif ($m =~ /^off$/i) { $number_flag = 0; }
				$n =~ s/\//\_/g;
				if ($n eq "") {
					$noname_num++;
					$n = "noname_${noname_num}";
				}
				$name = "note_${note_section_num}_$n";
				if (exists($note{$name})) {
					$notenum = $note{$name};
				} else {
					$notenum++;
					$note{$name} = $notenum;
				}
				if ($number_flag) {
					if ($note{$name} ne "") {
						$num = "(*$note{$name})";
					} else {
						$num = "(**)";
					}
				} else {
					$num = "";
				}
				if ($title_flag) { $num = "$note_string$num"; }
				unless (s/\Q$tag/<li><a name=\"$name\">$num<\/a>\&nbsp\;/i) {
					die "line$line: fail to parse: $_\n";
				}
			}
# This is for temporary files. But, temporary files are not used.
#			print WFILE $_;
#			unless ($_ = <RFILE>) { last; }
			push(@lines2, $_);
			push(@note_toc, $_);
			unless ($_ = shift(@lines)) { last; }
		}
		s/<\/notes>/<\/ul>/i;
		push(@note_toc, $_);
		$notenum = 0;
		$note_section_num++;
		%note = ();
	} else {
		while (($tag) = /(<noteref[^>]*?>)/i) {
			($t) = $tag =~ /<noteref[^>]*?title\s*=\s*([^\s>]*)[\s>]/i;
			if ($t =~ /\".*\"/) { ($t) = $t =~ /\"(.*)\"/; }
			($n) = $tag =~ /<noteref[^>]*?name\s*=\s*([^\s>]*)[\s>]/i;
			if ($n =~ /\".*\"/) { ($n) = $n =~ /\"(.*)\"/; }
			($m) = $tag =~ /<noteref[^>]*?number\s*=\s*([^\s>]*)[\s>]/i;
			if ($m =~ /\".*\"/) { ($m) = $m =~ /\"(.*)\"/; }
			$title_flag = 0;
			if    ($t =~ /^yes$/i) { $title_flag = 1; }
			elsif ($t =~ /^on$/i ) { $title_flag = 1; }
			elsif ($t =~ /^no$/i ) { $title_flag = 0; }
			elsif ($t =~ /^off$/i) { $title_flag = 0; }
			$number_flag = $note_number_flag;
			if    ($m =~ /^yes$/i) { $number_flag = 1; }
			elsif ($m =~ /^on$/i ) { $number_flag = 1; }
			elsif ($m =~ /^no$/i ) { $number_flag = 0; }
			elsif ($m =~ /^off$/i) { $number_flag = 0; }
			$n =~ s/\//\_/g;
			if ($n eq "") { die "line$line: $_:no name.\n"; }
			$name = "note_${note_section_num}_$n";
			if ($number_flag) {
				if (!exists($note{$name})) {
					$notenum++;
					$note{$name} = $notenum;
				}
				$num = "(*$note{$name})";
			} else {
				$note{$name} = "";
				$num = "";
			}
			if ($title_flag) { $num = "$note_string$num"; }
			unless (s/\Q$tag/<a href=\"\#$name\">$num<\/a>/i) {
				die "line$line: fail to parse: $_\n";
			}
		}
	}
# This is for temporary files. But, temporary files are not used.
#	print WFILE $_;
	push(@lines2, $_);
}

# This is for temporary files. But, temporary files are not used.
#close (RFILE);
#close (WFILE);
#open (RFILE, "< $filename.$filename_num");

if ($output_filename ne "") {
	open (WFILE, "> $output_filename") || die "cannot open: $output_filename\n";
}

$toc_string = "<h1>$toc_string<\/h1>\n\n$toc";
$table_index_string = "<h1>$table_index_string<\/h1>\n\n<ul>\n@table_toc<\/ul>";
$figure_index_string = "<h1>$figure_index_string<\/h1>\n\n<ul>\n@figure_toc<\/ul>";
$list_index_string = "<h1>$list_index_string<\/h1>\n\n<ul>\n@list_toc<\/ul>";
$reference_index_string = "<h2>$reference_index_string<\/h2>\n\n<ul>\n@ref_toc<\/ul>";
$term_index_string = "<h2>$term_index_string<\/h2>\n\n<ul>\n@term_toc<\/ul>";
$note_index_string = "<h2>$note_index_string<\/h2>\n\n<ul>\n@note_toc<\/ul>";

# This is for temporary files. But, temporary files are not used.
#while (<RFILE>)
while ($_ = shift(@lines2))
{
	if (/<tableofcontents>.*<\/tableofcontents>/i) {
		s/<tableofcontents>.*?<\/tableofcontents>/$toc_string/i;
	}
	if (/<tableindex>.*<\/tableindex>/i) {
		s/<tableindex>.*?<\/tableindex>/$table_index_string/i;
	}
	if (/<figureindex>.*<\/figureindex>/i) {
		s/<figureindex>.*?<\/figureindex>/$figure_index_string/i;
	}
	if (/<listindex>.*<\/listindex>/i) {
		s/<listindex>.*?<\/listindex>/$list_index_string/i;
	}
	if ($output_filename ne "") {
		print WFILE $_;
	} else {
		print $_;
	}
}

# This is for temporary files. But, temporary files are not used.
#close (RFILE);

if ($output_filename ne "") {
	close (WFILE);
}

if ($toc_filename ne "") {
	open (WFILE, "> $toc_filename");
	print WFILE "$toc_string";
	close (WFILE);
}
if ($table_filename ne "") {
	open (WFILE, "> $table_filename");
	print WFILE "$table_index_string";
	close (WFILE);
}
if ($figure_filename ne "") {
	open (WFILE, "> $figure_filename");
	print WFILE "$figure_index_string";
	close (WFILE);
}
if ($list_filename ne "") {
	open (WFILE, "> $list_filename");
	print WFILE "$list_index_string";
	close (WFILE);
}
if ($ref_filename ne "") {
	open (WFILE, "> $ref_filename");
	print WFILE "$reference_index_string";
	close (WFILE);
}
if ($term_filename ne "") {
	open (WFILE, "> $term_filename");
	print WFILE "$term_index_string";
	close (WFILE);
}
if ($note_filename ne "") {
	open (WFILE, "> $note_filename");
	print WFILE "$note_index_string";
	close (WFILE);
}

# This is for temporary files. But, temporary files are not used.
#system ("rm -f $filename.*");

exit (0);

###############################################################################
# HTMLSECTION_VERSION
